Package: polyCub
Title: Cubature over Polygonal Domains
Version: 0.9.4
Date: 2026-04-24
Authors@R: c(
    person("Sebastian", "Meyer",
           email = "seb.meyer@fau.de",
           role = c("aut","cre","trl"),
           comment = c(ORCID = "0000-0002-1791-9449")),
    person("Leonhard", "Held", role = "ths"),
    person("Michael", "Hoehle", role = "ths")
    )
Description: Numerical integration of continuously differentiable
    functions f(x,y) over simple closed polygonal domains.
    The following cubature methods are implemented:
    product Gauss cubature (Sommariva and Vianello, 2007,
    <doi:10.1007/s10543-007-0131-2>),
    the simple two-dimensional midpoint rule
    (wrapping 'spatstat.geom' functions), and
    adaptive cubature for radially symmetric functions via line
    integrate() along the polygon boundary (Meyer and Held, 2014,
    <doi:10.1214/14-AOAS743>, Supplement B).
    For simple integration along the axes, the 'cubature' package
    is more appropriate.
License: GPL-2
URL: https://github.com/bastistician/polyCub
BugReports: https://github.com/bastistician/polyCub/issues
Note: Building the package requires R >= 4.6.0 for \bibshow{} et al.
Depends: R (>= 3.4.0), methods
Imports: grDevices, graphics, stats, sp (>= 1.0-11)
Suggests: spatstat.geom, lattice, mvtnorm, statmod, sf, cubature,
        litedown (>= 0.9), microbenchmark
VignetteBuilder: litedown
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-24 09:20:44 UTC; smeyer
Author: Sebastian Meyer [aut, cre, trl] (ORCID:
    <https://orcid.org/0000-0002-1791-9449>),
  Leonhard Held [ths],
  Michael Hoehle [ths]
Maintainer: Sebastian Meyer <seb.meyer@fau.de>
Repository: CRAN
Date/Publication: 2026-04-24 10:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 15:39:58 UTC; unix
Archs: polyCub.so.dSYM
