# Generated by roxygen2: do not edit by hand

S3method(xylist,Polygon)
S3method(xylist,Polygons)
S3method(xylist,SpatialPolygons)
S3method(xylist,default)
S3method(xylist,gpc.poly)
S3method(xylist,owin)
S3method(xylist,sfg)
export(.polyCub.iso)
export(as.owin.Polygon)
export(as.owin.Polygons)
export(as.owin.SpatialPolygons)
export(as.owin.gpc.poly)
export(checkintrfr)
export(circleCub.Gauss)
export(gpc2owin)
export(owin2gpc)
export(plotpolyf)
export(polyCub)
export(polyCub.SV)
export(polyCub.exact.Gauss)
export(polyCub.iso)
export(polyCub.midpoint)
export(sfg2gpc)
export(xylist)
exportMethods(coerce)
if(getRversion() >= "3.6.0") {  # delayed registration
    S3method(spatstat.geom::as.owin, SpatialPolygons)
    S3method(spatstat.geom::as.owin, Polygons)
    S3method(spatstat.geom::as.owin, Polygon)
}
if(getRversion() >= "3.6.0") {  # delayed registration
    S3method(spatstat.geom::as.owin, gpc.poly)
}
import(methods)
importClassesFrom(sp,Polygon)
importClassesFrom(sp,Polygons)
importClassesFrom(sp,SpatialPolygons)
importClassesFrom(sp,owin)
importFrom(grDevices,extendrange)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,xy.coords)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(sp,coordinates)
importFrom(sp,plot)
importFrom(stats,cov2cor)
importFrom(stats,dist)
importFrom(stats,integrate)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importMethodsFrom(sp,plot)
useDynLib(polyCub, .registration = TRUE)
