bibentry(
    key = "meyer.held2014",
    bibtype = "Article",
    author = c(person("Sebastian", "Meyer"),
               person("Leonhard", "Held")),
    title = "Power-law models for infectious disease spread",
    journal = "Annals of Applied Statistics",
    year = "2014",
    volume = "8",
    number = "3",
    pages = "1612--1639",
    doi = "10.1214/14-AOAS743"
)

bibentry(
    key = "sommariva.vianello2007",
    bibtype = "Article",
    author = c(person(given = "Alvise", family = "Sommariva"),
               person(given = "Marco", family = "Vianello")),
    title = "Product {Gauss} cubature over polygons based on {Green}'s integration formula",
    journal = "BIT Numerical Mathematics",
    year = "2007",
    volume = "47",
    number = "2",
    pages = "441--453",
    doi = "10.1007/s10543-007-0131-2"
)

bibentry(
    key = "meyer2010",
    bibtype = "MastersThesis",
    author = "Sebastian Meyer",
    school = "Department of Statistics, LMU",
    title = "Spatio-Temporal Infectious Disease Epidemiology based on Point Processes",
    year = "2010",
    address = "Munich, Germany",
    url = "https://epub.ub.uni-muenchen.de/11703/"
)

bibentry(
    key = "R:spatstat.geom",
    bibtype = "Book",
    title = "Spatial Point Patterns: Methodology and Applications with {R}",
    author = "Adrian Baddeley and Ege Rubak and Rolf Turner",
    year = "2015",
    publisher = "Chapman and Hall/CRC Press",
    address = "London",
    isbn = "9781482210200"
)
