try(citation("polyCub"), silent = TRUE) # OK for R CMD build/INSTALL/check
## NOTE: Requires installation, so is incompatible with building directly
##       from source via `R CMD Rd2pdf pkgdir` or `tools::pkg2HTML(dir=)`.
##       A more general approach would be:
##   pkgdir <- tools:::Rd_macros_package_dir()
##   if (file.exists(descfile <- file.path(pkgdir, "DESCRIPTION")) && # not for base
##       read.dcf(descfile, "Package") == "polyCub") {
##       readCitationFile(file.path(pkgdir, "inst", "CITATION"))
##   } else { # foreign use of 'polyCub::CITATION': access installed package
##       citation("polyCub")
##   }

bibentry(
    key = "meyer.held2014",
    bibtype = "Article",
    author = c(person(given = "Sebastian", family = "Meyer"),
               person(given = "Leonhard", family = "Held")),
    title = "Power-law models for infectious disease spread",
    journal = "Annals of Applied Statistics",
    year = "2014",
    volume = "8",
    number = "3",
    pages = "1612--1639",
    doi = "10.1214/14-AOAS743"
)

bibentry(
    key = "sommariva.vianello2007",
    bibtype = "Article",
    author = c(person(given = "Alvise", family = "Sommariva"),
               person(given = "Marco", family = "Vianello")),
    title = "Product {Gauss} cubature over polygons based on {Green}'s integration formula",
    journal = "BIT Numerical Mathematics",
    year = "2007",
    volume = "47",
    number = "2",
    pages = "441--453",
    doi = "10.1007/s10543-007-0131-2"
)

bibentry(
    key = "meyer2010",
    bibtype = "MastersThesis",
    author = person(given = "Sebastian", family = "Meyer"),
    school = "Department of Statistics, LMU",
    title = "Spatio-Temporal Infectious Disease Epidemiology based on Point Processes",
    year = "2010",
    address = "Munich, Germany",
    url = "https://epub.ub.uni-muenchen.de/11703/"
)
